@@ -45,6 +45,7 @@
 #include "openvswitch/ofp-actions.h"
 #include "openvswitch/ofp-bundle.h"
 #include "openvswitch/ofp-connection.h"
+#include "openvswitch/ofp-ct.h"
 #include "openvswitch/ofp-errors.h"
 #include "openvswitch/ofp-group.h"
 #include "openvswitch/ofp-ipfix.h"
@@ -744,10 +745,10 @@ ofp_print_nxt_flow_monitor_cancel(struct ds *string,
 }
 
 static enum ofperr
-ofp_print_nxst_flow_monitor_request(struct ds *string,
-                                    const struct ofp_header *oh,
-                                    const struct ofputil_port_map *port_map,
-                                    const struct ofputil_table_map *table_map)
+ofp_print_flow_monitor_request(struct ds *string,
+                               const struct ofp_header *oh,
+                               const struct ofputil_port_map *port_map,
+                               const struct ofputil_table_map *table_map)
 {
     struct ofpbuf b = ofpbuf_const_initializer(oh, ntohs(oh->length));
     for (;;) {
@@ -765,10 +766,10 @@ ofp_print_nxst_flow_monitor_request(struct ds *string,
 }
 
 static enum ofperr
-ofp_print_nxst_flow_monitor_reply(struct ds *string,
-                                  const struct ofp_header *oh,
-                                  const struct ofputil_port_map *port_map,
-                                  const struct ofputil_table_map *table_map)
+ofp_print_flow_monitor_reply(struct ds *string,
+                             const struct ofp_header *oh,
+                             const struct ofputil_port_map *port_map,
+                             const struct ofputil_table_map *table_map)
 {
     uint64_t ofpacts_stub[1024 / 8];
     struct ofpbuf ofpacts = OFPBUF_STUB_INITIALIZER(ofpacts_stub);
@@ -949,6 +950,23 @@ ofp_print_nxt_ct_flush_zone(struct ds *string, const struct nx_zone_id *nzi)
     return 0;
 }
 
+static enum ofperr
+ofp_print_nxt_ct_flush(struct ds *string, const struct ofp_header *oh)
+{
+    uint16_t zone_id = 0;
+    struct ofp_ct_match match = {0};
+
+    enum ofperr error = ofp_ct_match_decode(&match, NULL, &zone_id, oh);
+    if (error) {
+        return error;
+    }
+
+    ds_put_format(string, " zone=%"PRIu16" ", zone_id);
+    ofp_ct_match_format(string, &match);
+
+    return 0;
+}
+
 static enum ofperr
 ofp_to_string__(const struct ofp_header *oh,
                 const struct ofputil_port_map *port_map,
@@ -1147,12 +1165,12 @@ ofp_to_string__(const struct ofp_header *oh,
         break;
 
     case OFPTYPE_FLOW_MONITOR_STATS_REQUEST:
-        return ofp_print_nxst_flow_monitor_request(string, msg, port_map,
-                                                   table_map);
+        return ofp_print_flow_monitor_request(string, msg, port_map,
+                                              table_map);
 
     case OFPTYPE_FLOW_MONITOR_STATS_REPLY:
-        return ofp_print_nxst_flow_monitor_reply(string, msg, port_map,
-                                                 table_map);
+        return ofp_print_flow_monitor_reply(string, msg, port_map,
+                                            table_map);
 
     case OFPTYPE_BUNDLE_CONTROL:
         return ofp_print_bundle_ctrl(string, msg);
@@ -1184,6 +1202,8 @@ ofp_to_string__(const struct ofp_header *oh,
 
     case OFPTYPE_CT_FLUSH_ZONE:
         return ofp_print_nxt_ct_flush_zone(string, ofpmsg_body(oh));
+    case OFPTYPE_CT_FLUSH:
+        return ofp_print_nxt_ct_flush(string, oh);
     }
 
     return 0;
