@@ -27,6 +27,7 @@
 #include <string.h>
 #ifdef __linux__
 #include <sys/prctl.h>
+#include <sys/utsname.h>
 #endif
 #include <sys/stat.h>
 #include <unistd.h>
@@ -66,8 +67,8 @@ DEFINE_PER_THREAD_MALLOCED_DATA(char *, subprogram_name);
 /* --version option output. */
 static char *program_version;
 
-/* 'true' if mlockall() succeeded. */
-static bool is_memory_locked = false;
+/* 'true' if mlockall() succeeded, but doesn't support ONFAULT. */
+static bool is_all_memory_locked = false;
 
 /* Buffer used by ovs_strerror() and ovs_format_message(). */
 DEFINE_STATIC_PER_THREAD_DATA(struct { char s[128]; },
@@ -101,15 +102,15 @@ ovs_assert_failure(const char *where, const char *function,
 }
 
 void
-set_memory_locked(void)
+set_all_memory_locked(void)
 {
-    is_memory_locked = true;
+    is_all_memory_locked = true;
 }
 
 bool
-memory_locked(void)
+memory_all_locked(void)
 {
-    return is_memory_locked;
+    return is_all_memory_locked;
 }
 
 void
@@ -619,12 +620,14 @@ ovs_set_program_name(const char *argv0, const char *version)
     program_name = basename;
 
     free(program_version);
-    if (!strcmp(version, VERSION)) {
-        program_version = xasprintf("%s (Open vSwitch) "VERSION"\n",
+    if (!strcmp(version, VERSION VERSION_SUFFIX)) {
+        program_version = xasprintf("%s (Open vSwitch) "VERSION
+                                    VERSION_SUFFIX,
                                     program_name);
     } else {
         program_version = xasprintf("%s %s\n"
-                                    "Open vSwitch Library "VERSION"\n",
+                                    "Open vSwitch Library "VERSION
+                                    VERSION_SUFFIX,
                                     program_name, version);
     }
 }
@@ -761,7 +764,7 @@ ovs_get_program_name(void)
 void
 ovs_print_version(uint8_t min_ofp, uint8_t max_ofp)
 {
-    printf("%s", program_version);
+    printf("%s\n", program_version);
     if (min_ofp || max_ofp) {
         printf("OpenFlow versions %#x:%#x\n", min_ofp, max_ofp);
     }
@@ -2502,3 +2505,29 @@ OVS_CONSTRUCTOR(winsock_start) {
    }
 }
 #endif
+
+#ifdef __linux__
+bool
+ovs_kernel_is_version_or_newer(int target_major, int target_minor)
+{
+    static struct ovsthread_once once = OVSTHREAD_ONCE_INITIALIZER;
+    static int current_major, current_minor = -1;
+
+    if (ovsthread_once_start(&once)) {
+        struct utsname utsname;
+
+        if (uname(&utsname) == -1) {
+            VLOG_WARN("uname failed (%s)", ovs_strerror(errno));
+        } else if (!ovs_scan(utsname.release, "%d.%d",
+                    &current_major, &current_minor)) {
+            VLOG_WARN("uname reported bad OS release (%s)", utsname.release);
+        }
+        ovsthread_once_done(&once);
+    }
+    if (current_major == -1 || current_minor == -1) {
+        return false;
+    }
+    return current_major > target_major || (
+            current_major == target_major && current_minor >= target_minor);
+}
+#endif
