@@ -20,6 +20,7 @@
 #include <limits.h>
 #include <string.h>
 
+#include "cooperative-multitasking.h"
 #include "file.h"
 #include "openvswitch/json.h"
 #include "jsonrpc.h"
@@ -31,6 +32,7 @@
 #include "transaction-forward.h"
 #include "openvswitch/vlog.h"
 #include "util.h"
+#include "uuid.h"
 
 VLOG_DEFINE_THIS_MODULE(trigger);
 
@@ -52,6 +54,7 @@ ovsdb_trigger_init(struct ovsdb_session *session, struct ovsdb *db,
     trigger->db = db;
     ovs_list_push_back(&trigger->db->triggers, &trigger->node);
     trigger->request = request;
+    trigger->converted_db = NULL;
     trigger->reply = NULL;
     trigger->progress = NULL;
     trigger->txn_forward = NULL;
@@ -69,6 +72,7 @@ ovsdb_trigger_destroy(struct ovsdb_trigger *trigger)
     ovsdb_txn_progress_destroy(trigger->progress);
     ovsdb_txn_forward_destroy(trigger->db, trigger->txn_forward);
     ovs_list_remove(&trigger->node);
+    ovsdb_destroy(trigger->converted_db);
     jsonrpc_msg_destroy(trigger->request);
     jsonrpc_msg_destroy(trigger->reply);
     free(trigger->role);
@@ -143,6 +147,30 @@ ovsdb_trigger_prereplace_db(struct ovsdb_trigger *trigger)
     }
 }
 
+/* Find among incomplete triggers one that caused database conversion
+ * with specified transaction ID. */
+struct ovsdb *
+ovsdb_trigger_find_and_steal_converted_db(const struct ovsdb *db,
+                                          const struct uuid *txnid)
+{
+    struct ovsdb *converted_db = NULL;
+    struct ovsdb_trigger *t;
+
+    if (uuid_is_zero(txnid)) {
+        return NULL;
+    }
+
+    LIST_FOR_EACH_SAFE (t, node, &db->triggers) {
+        if (t->db == db && t->converted_db
+            && uuid_equals(&t->conversion_txnid, txnid)) {
+            converted_db = t->converted_db;
+            t->converted_db = NULL;
+            break;
+        }
+    }
+    return converted_db;
+}
+
 bool
 ovsdb_trigger_run(struct ovsdb *db, long long int now)
 {
@@ -154,6 +182,8 @@ ovsdb_trigger_run(struct ovsdb *db, long long int now)
     bool disconnect_all = false;
 
     LIST_FOR_EACH_SAFE (t, node, &db->triggers) {
+        cooperative_multitasking_yield();
+
         if (run_triggers
             || now - t->created >= t->timeout_msec
             || t->progress || t->txn_forward) {
@@ -200,7 +230,6 @@ ovsdb_trigger_try(struct ovsdb_trigger *t, long long int now)
         ovs_assert(!t->progress);
 
         struct ovsdb_txn *txn = NULL;
-        struct ovsdb *newdb = NULL;
         if (!strcmp(t->request->method, "transact")) {
             if (!ovsdb_txn_precheck_prereq(t->db)) {
                 return false;
@@ -252,6 +281,14 @@ ovsdb_trigger_try(struct ovsdb_trigger *t, long long int now)
                 return false;
             }
 
+            if (t->read_only) {
+                trigger_convert_error(
+                    t, ovsdb_error("not allowed", "conversion is not allowed "
+                                                  "for read-only database %s",
+                                                  t->db->schema->name));
+                return false;
+            }
+
             /* Validate parameters. */
             const struct json *params = t->request->params;
             if (params->type != JSON_ARRAY || params->array.n != 2) {
@@ -272,7 +309,8 @@ ovsdb_trigger_try(struct ovsdb_trigger *t, long long int now)
                                     new_schema->name, t->db->schema->name);
             }
             if (!error) {
-                error = ovsdb_convert(t->db, new_schema, &newdb);
+                ovsdb_destroy(t->converted_db);
+                error = ovsdb_convert(t->db, new_schema, &t->converted_db);
             }
             if (error) {
                 ovsdb_schema_destroy(new_schema);
@@ -280,13 +318,18 @@ ovsdb_trigger_try(struct ovsdb_trigger *t, long long int now)
                 return false;
             }
 
-            /* Make the new copy into a transaction log record. */
-            struct json *txn_json = ovsdb_to_txn_json(
-                newdb, "converted by ovsdb-server", true);
+            struct json *txn_json;
+            if (ovsdb_conversion_with_no_data_supported(t->db)) {
+                txn_json = json_null_create();
+            } else {
+                /* Make the new copy into a transaction log record. */
+                txn_json = ovsdb_to_txn_json(
+                    t->converted_db, "converted by ovsdb-server", true);
+            }
 
             /* Propose the change. */
             t->progress = ovsdb_txn_propose_schema_change(
-                t->db, new_schema, txn_json);
+                t->db, new_schema, txn_json, &t->conversion_txnid);
             ovsdb_schema_destroy(new_schema);
             json_destroy(txn_json);
             t->reply = jsonrpc_create_reply(json_object_create(),
@@ -308,13 +351,13 @@ ovsdb_trigger_try(struct ovsdb_trigger *t, long long int now)
             ovsdb_txn_progress_destroy(t->progress);
             t->progress = NULL;
             ovsdb_trigger_complete(t);
-            if (newdb) {
-                ovsdb_replace(t->db, newdb);
+            if (t->converted_db) {
+                ovsdb_replace(t->db, t->converted_db);
+                t->converted_db = NULL;
                 return true;
             }
             return false;
         }
-        ovsdb_destroy(newdb);
 
         /* Fall through to the general handling for the "committing" state.  We
          * abort the transaction--if and when it eventually commits, we'll read
