@@ -508,6 +508,8 @@ def idl_set(idl, commands, step):
 
             s = txn.insert(idl.tables["simple"])
             s.i = int(args[0])
+        elif name == "insert_no_columns_changed":
+            txn.insert(idl.tables["simple"])
         elif name == "insert_uuid":
             if len(args) != 2:
                 sys.stderr.write('"set" command requires 2 argument\n')
@@ -516,6 +518,17 @@ def idl_set(idl, commands, step):
             s = txn.insert(idl.tables["simple"], new_uuid=uuid.UUID(args[0]),
                            persist_uuid=True)
             s.i = int(args[1])
+        elif name == "insert_uuid_uref":
+            if len(args) != 2:
+                sys.stderr.write('"set" command requires 2 argument\n')
+                sys.exit(1)
+
+            s4 = txn.insert(idl.tables["simple4"], new_uuid=uuid.UUID(args[1]),
+                            persist_uuid=True)
+            s3 = txn.insert(idl.tables["simple3"], new_uuid=uuid.UUID(args[0]),
+                            persist_uuid=True)
+            s3.uref = s4
+
         elif name == "add_op":
             if len(args) != 1:
                 sys.stderr.write('"add_op" command requires 1 argument\n')
@@ -745,7 +758,7 @@ def do_idl(schema_file, remote, *commands):
 
     if remote.startswith("ssl:"):
         if len(commands) < 3:
-            sys.stderr.write("SSL connection requires private key, "
+            sys.stderr.write("SSL/TLS connection requires private key, "
                              "certificate for private key, and peer CA "
                              "certificate as arguments\n")
             sys.exit(1)
@@ -970,7 +983,7 @@ def do_idl_cluster(schema_file, remote, pid, *commands):
 
     if remote.startswith("ssl:"):
         if len(commands) < 3:
-            sys.stderr.write("SSL connection requires private key, "
+            sys.stderr.write("SSL/TLS connection requires private key, "
                              "certificate for private key, and peer CA "
                              "certificate as arguments\n")
             sys.exit(1)
