@@ -1116,12 +1116,16 @@ test_snprintf(struct ovs_cmdl_context *ctx OVS_UNUSED)
 {
     char s[16];
 
+    /* GCC 7+ and Clang 18+ warn about the following calls that truncate
+     * a string using snprintf().  We're testing that truncation works
+     * properly, so temporarily disable the warning. */
 #if __GNUC__ >= 7
-    /* GCC 7+ warns about the following calls that truncate a string using
-     * snprintf().  We're testing that truncation works properly, so
-     * temporarily disable the warning. */
 #pragma GCC diagnostic push
 #pragma GCC diagnostic ignored "-Wformat-truncation"
+#endif
+#if __clang_major__ >= 18
+#pragma clang diagnostic push
+#pragma clang diagnostic ignored "-Wformat-truncation"
 #endif
     ovs_assert(snprintf(s, 4, "abcde") == 5);
     ovs_assert(!strcmp(s, "abc"));
@@ -1130,6 +1134,9 @@ test_snprintf(struct ovs_cmdl_context *ctx OVS_UNUSED)
     ovs_assert(!strcmp(s, "abcd"));
 #if __GNUC__ >= 7
 #pragma GCC diagnostic pop
+#endif
+#if __clang_major__ >= 18
+#pragma clang diagnostic pop
 #endif
 
     ovs_assert(snprintf(s, 6, "abcde") == 5);
